@@ -148,12 +148,7 @@ static const char* getProcessNameAndPpid(pid_t pid, char* name, pid_t* ppid, int
         return "sscanf(stat) failed";
 
     if (tty)
-    {
-        if ((tty_ >> 8) == 0x88)
-            *tty = tty_ & 0xFF;
-        else
-            *tty = -1;
-    }
+        *tty = tty_ & 0xFF;
 
     #elif defined(__APPLE__)
 
@@ -571,6 +566,7 @@ const FFTerminalResult* ffDetectTerminal()
     result.exeName = result.exe.chars;
     ffStrbufInit(&result.exePath);
     ffStrbufInit(&result.version);
+    ffStrbufInitS(&result.tty, ttyname(STDOUT_FILENO));
     result.pid = 0;
     result.ppid = 0;
 
