@@ -2,87 +2,201 @@
 #include "common/io/io.h"
 #include "common/printing.h"
 
+#include <limits.h>
+#include <math.h>
+
 #ifdef __APPLE__
     #include <sys/syslimits.h>
 #elif _WIN32
     #include <windows.h>
 #endif
 
-static FFstrbuf base64Encode(FFstrbuf* in)
+// https://github.com/kostya/benchmarks/blob/master/base64/test-nolib.c#L145
+static void base64EncodeRaw(uint32_t size, const char *str, uint32_t *out_size, char *output)
 {
-    const char* base64Chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
-
-    FFstrbuf out;
-    ffStrbufInitA(&out, 8 * (1 + in->length / 6));
-
-    unsigned val = 0;
-    int valb = -6;
-    for (uint32_t i = 0; i < in->length; ++i)
-    {
-        unsigned char c = (unsigned char) in->chars[i];
-        val = (val << 8) + c;
-        valb += 8;
-        while (valb >= 0)
-        {
-            ffStrbufAppendC(&out, base64Chars[(val>>valb)&0x3F]);
-            valb -= 6;
-        }
+    static const char chars[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+    char *out = output;
+    const char *ends = str + (size - size % 3);
+    while (str != ends) {
+        uint32_t n = __builtin_bswap32(*(uint32_t*) str);
+        *out++ = chars[(n >> 26) & 63];
+        *out++ = chars[(n >> 20) & 63];
+        *out++ = chars[(n >> 14) & 63];
+        *out++ = chars[(n >> 8) & 63];
+        str += 3;
     }
-    if (valb > -6) ffStrbufAppendC(&out, base64Chars[((val<<8)>>(valb+8))&0x3F]);
-    while (out.length % 4) ffStrbufAppendC(&out, '=');
+
+    if (size % 3 == 1) {
+        uint64_t n = (uint64_t)*str << 16;
+        *out++ = chars[(n >> 18) & 63];
+        *out++ = chars[(n >> 12) & 63];
+        *out++ = '=';
+        *out++ = '=';
+    } else if (size % 3 == 2) {
+        uint64_t n = (uint64_t)*str++ << 16;
+        n |= (uint64_t)*str << 8;
+        *out++ = chars[(n >> 18) & 63];
+        *out++ = chars[(n >> 12) & 63];
+        *out++ = chars[(n >> 6) & 63];
+        *out++ = '=';
+    }
+    *out = '\0';
+    *out_size = (uint32_t) (out - output);
+}
+
+static FFstrbuf base64Encode(const FFstrbuf* in)
+{
+    FFstrbuf out = ffStrbufCreateA(10 + in->length * 4 / 3);
+    base64EncodeRaw(in->length, in->chars, &out.length, out.chars);
+    assert(out.length < out.allocated);
+
     return out;
 }
 
-static bool printImageIterm(FFinstance* instance)
+static bool printImageIterm(void)
 {
-    if(instance->config.logo.width == 0 || instance->config.logo.height == 0)
-    {
-        fputs("Logo: `iterm` protocol only works when both `--logo-width` and `--logo-height` being specified\n", stderr);
-        return false;
-    }
-
-    FFstrbuf buf;
-    ffStrbufInit(&buf);
-    if(!ffAppendFileBuffer(instance->config.logo.source.chars, &buf))
+    const FFOptionsLogo* options = &instance.config.logo;
+    FF_STRBUF_AUTO_DESTROY buf = ffStrbufCreate();
+    if(!ffAppendFileBuffer(options->source.chars, &buf))
     {
         fputs("Logo: Failed to load image file\n", stderr);
         return false;
     }
 
-    ffPrintCharTimes(' ', instance->config.logo.paddingLeft);
-    ffPrintCharTimes('\n', instance->config.logo.paddingTop);
-    FFstrbuf base64 = base64Encode(&buf);
-    instance->state.logoWidth = instance->config.logo.width + instance->config.logo.paddingLeft + instance->config.logo.paddingRight;
-    instance->state.logoHeight = instance->config.logo.paddingTop + instance->config.logo.height;
-    printf("\033]1337;File=inline=1;width=%u;height=%u;preserveAspectRatio=%u:%s\a\033[9999999D\n\033[%uA",
-        (unsigned) instance->config.logo.width,
-        (unsigned) instance->config.logo.height,
-        (unsigned) instance->config.logo.preserveAspectRadio,
-        base64.chars,
-        (unsigned) instance->state.logoHeight
-    );
+    fflush(stdout);
 
-    ffStrbufDestroy(&buf);
-    ffStrbufDestroy(&base64);
+    FF_STRBUF_AUTO_DESTROY base64 = base64Encode(&buf);
+    ffStrbufClear(&buf);
+
+    if (!options->width || !options->height)
+    {
+        if (!options->separate)
+        {
+            ffStrbufAppendF(&buf, "\e[2J\e[3J\e[%u;%uH",
+                (unsigned) options->paddingTop,
+                (unsigned) options->paddingLeft
+            );
+        }
+        else
+        {
+            ffStrbufAppendNC(&buf, options->paddingTop, '\n');
+            ffStrbufAppendNC(&buf, options->paddingLeft, ' ');
+        }
+        ffStrbufAppendF(&buf, "\e]1337;File=inline=1:%s\a",
+            base64.chars
+        );
+        ffWriteFDBuffer(FFUnixFD2NativeFD(STDOUT_FILENO), &buf);
+
+        if (!options->separate)
+        {
+            uint16_t X = 0, Y = 0;
+            const char* error = ffGetTerminalResponse("\e[6n", "\e[%hu;%huR", &Y, &X);
+            if (error)
+            {
+                fprintf(stderr, "\nLogo (iterm): fail to query cursor position: %s\n", error);
+                return true; // We already printed image logo, don't print ascii logo then
+            }
+            instance.state.logoWidth = X + options->paddingRight;
+            instance.state.logoHeight = Y;
+            fputs("\e[H", stdout);
+        }
+        else
+        {
+            instance.state.logoWidth = instance.state.logoHeight = 0;
+            ffPrintCharTimes('\n', options->paddingRight);
+        }
+    }
+    else
+    {
+        ffStrbufAppendNC(&buf, options->paddingTop, '\n');
+        ffStrbufAppendNC(&buf, options->paddingLeft, ' ');
+        ffStrbufAppendF(&buf, "\e]1337;File=inline=1;width=%u;height=%u;preserveAspectRatio=%u:%s\a\n",
+            (unsigned) options->width,
+            (unsigned) options->height,
+            (unsigned) options->preserveAspectRatio,
+            base64.chars
+        );
+
+        if (!options->separate)
+        {
+            instance.state.logoWidth = options->width + options->paddingLeft + options->paddingRight;
+            instance.state.logoHeight = options->paddingTop + options->height;
+            ffStrbufAppendF(&buf, "\e[%uA", (unsigned) instance.state.logoHeight);
+        }
+        else
+        {
+            instance.state.logoWidth = instance.state.logoHeight = 0;
+            ffStrbufAppendNC(&buf, options->paddingRight, '\n');
+        }
+        ffWriteFDBuffer(FFUnixFD2NativeFD(STDOUT_FILENO), &buf);
+    }
 
     return true;
 }
 
-static bool printImageKittyDirect(FFinstance* instance)
+static bool printImageKittyDirect(void)
 {
-    ffPrintCharTimes(' ', instance->config.logo.paddingLeft);
-    ffPrintCharTimes('\n', instance->config.logo.paddingTop);
-    FFstrbuf base64 = base64Encode(&instance->config.logo.source);
-    printf("\033_Ga=T,f=100,t=f,c=%u,r=%u,C=1;%s\033\\\033[9999999D",
-        (unsigned) instance->config.logo.width,
-        (unsigned) instance->config.logo.height,
-        base64.chars
-    );
+    const FFOptionsLogo* options = &instance.config.logo;
+    FF_STRBUF_AUTO_DESTROY base64 = base64Encode(&options->source);
 
-    instance->state.logoWidth = instance->config.logo.width + instance->config.logo.paddingLeft + instance->config.logo.paddingRight;
-    instance->state.logoHeight = instance->config.logo.paddingTop + instance->config.logo.height;
+    if (!options->width || !options->height)
+    {
+        if (!options->separate)
+        {
+            // We must clear the entre screen to make sure that terminal buffer won't scroll up
+            printf("\e[2J\e[3J\e[%u;%uH",
+                (unsigned) options->paddingTop,
+                (unsigned) options->paddingLeft
+            );
+        }
+        else
+        {
+            ffPrintCharTimes('\n', options->paddingTop);
+            ffPrintCharTimes(' ', options->paddingLeft);
+        }
+        printf("\e_Ga=T,f=100,t=f;%s\e\\", base64.chars);
+        fflush(stdout);
+        if (!options->separate)
+        {
+            uint16_t X = 0, Y = 0;
+            const char* error = ffGetTerminalResponse("\e[6n", "\e[%hu;%huR", &Y, &X);
+            if (error)
+            {
+                fprintf(stderr, "\nLogo (kitty-direct): fail to query cursor position: %s\n", error);
+                return true; // We already printed image logo, don't print ascii logo then
+            }
+            instance.state.logoWidth = X + options->paddingRight;
+            instance.state.logoHeight = Y;
+            fputs("\e[H", stdout);
+        }
+        else
+        {
+            instance.state.logoWidth = instance.state.logoHeight = 0;
+            ffPrintCharTimes('\n', options->paddingRight);
+        }
+    }
+    else
+    {
+        ffPrintCharTimes('\n', options->paddingTop);
+        ffPrintCharTimes(' ', options->paddingLeft);
 
-    ffStrbufDestroy(&base64);
+        printf("\e_Ga=T,f=100,t=f,c=%u,r=%u;%s\e\\\n",
+            (unsigned) options->width,
+            (unsigned) options->height,
+            base64.chars
+        );
+        if (!options->separate)
+        {
+            instance.state.logoWidth = options->width + options->paddingLeft + options->paddingRight;
+            instance.state.logoHeight = options->paddingTop + options->height;
+            printf("\e[%uA", (unsigned) instance.state.logoHeight);
+        }
+        else
+        {
+            instance.state.logoWidth = instance.state.logoHeight = 0;
+            ffPrintCharTimes('\n', options->paddingRight);
+        }
+    }
 
     return true;
 }
@@ -124,9 +238,9 @@ static inline char* realpath(const char* restrict file_name, char* restrict reso
 #include <stdlib.h>
 #include <zlib.h>
 
-static bool compressBlob(const FFinstance* instance, void** blob, size_t* length)
+static bool compressBlob(void** blob, size_t* length)
 {
-    FF_LIBRARY_LOAD(zlib, &instance->config.libZ, false, "libz" FF_LIBRARY_EXTENSION, 2)
+    FF_LIBRARY_LOAD(zlib, &instance.config.library.libZ, false, "libz" FF_LIBRARY_EXTENSION, 2)
     FF_LIBRARY_LOAD_SYMBOL(zlib, compressBound, false)
     FF_LIBRARY_LOAD_SYMBOL(zlib, compress2, false)
 
@@ -182,12 +296,6 @@ static inline bool checkAllocationResult(void* data, size_t length)
     return true;
 }
 
-static inline uint32_t simpleCeil(double value)
-{
-    uint32_t result = (uint32_t) value;
-    return value == (double) result ? result : result + 1;
-}
-
 static void writeCacheStrbuf(FFLogoRequestData* requestData, const FFstrbuf* value, const char* cacheFileName)
 {
     uint32_t cacheDirLength = requestData->cacheDir.length;
@@ -204,36 +312,33 @@ static void writeCacheUint32(FFLogoRequestData* requestData, uint32_t value, con
     writeCacheStrbuf(requestData, &content, cacheFileName);
 }
 
-static void printImagePixels(FFinstance* instance, FFLogoRequestData* requestData, const FFstrbuf* result, const char* cacheFileName)
+static void printImagePixels(FFLogoRequestData* requestData, const FFstrbuf* result, const char* cacheFileName)
 {
+    const FFOptionsLogo* options = &instance.config.logo;
     //Calculate character dimensions
-    instance->state.logoWidth = requestData->logoCharacterWidth + instance->config.logo.paddingLeft + instance->config.logo.paddingRight;
-
-    instance->state.logoHeight = requestData->logoCharacterHeight + instance->config.logo.paddingTop;
-    if(requestData->type == FF_LOGO_TYPE_IMAGE_KITTY)
-        instance->state.logoHeight -= 1;
+    instance.state.logoWidth = requestData->logoCharacterWidth + options->paddingLeft + options->paddingRight;
+    instance.state.logoHeight = requestData->logoCharacterHeight + options->paddingTop - 1;
 
     //Write cache files
     writeCacheStrbuf(requestData, result, cacheFileName);
 
-    if(instance->config.logo.width == 0)
-        writeCacheUint32(requestData, instance->state.logoWidth, FF_CACHE_FILE_WIDTH);
+    if(options->width == 0)
+        writeCacheUint32(requestData, requestData->logoCharacterWidth, FF_CACHE_FILE_WIDTH);
 
-    if(instance->config.logo.height == 0)
-        writeCacheUint32(requestData, instance->state.logoHeight, FF_CACHE_FILE_HEIGHT);
+    if(options->height == 0)
+        writeCacheUint32(requestData, requestData->logoCharacterHeight, FF_CACHE_FILE_HEIGHT);
 
     //Write result to stdout
-    ffPrintCharTimes('\n', instance->config.logo.paddingTop);
-    ffPrintCharTimes(' ', instance->config.logo.paddingLeft);
+    ffPrintCharTimes('\n', options->paddingTop);
+    ffPrintCharTimes(' ', options->paddingLeft);
     fflush(stdout);
     ffWriteFDBuffer(FFUnixFD2NativeFD(STDOUT_FILENO), result);
 
     //Go to upper left corner
-    fputs("\033[9999999D", stdout);
-    printf("\033[%uA", instance->state.logoHeight);
+    printf("\e[1G\e[%uA", instance.state.logoHeight);
 }
 
-static bool printImageSixel(FFinstance* instance, FFLogoRequestData* requestData, const ImageData* imageData)
+static bool printImageSixel(FFLogoRequestData* requestData, const ImageData* imageData)
 {
     imageData->ffCopyMagickString(imageData->imageInfo->magick, "SIXEL", 6);
 
@@ -246,7 +351,7 @@ static bool printImageSixel(FFinstance* instance, FFLogoRequestData* requestData
     result.chars = (char*) blob;
     result.length = (uint32_t) length;
 
-    printImagePixels(instance, requestData, &result, FF_CACHE_FILE_SIXEL);
+    printImagePixels(requestData, &result, FF_CACHE_FILE_SIXEL);
 
     free(blob);
     return true;
@@ -269,7 +374,7 @@ static void appendKittyChunk(FFstrbuf* result, const char** blob, size_t* length
     *blob += chunkSize;
 }
 
-static bool printImageKitty(FFinstance* instance, FFLogoRequestData* requestData, const ImageData* imageData)
+static bool printImageKitty(FFLogoRequestData* requestData, const ImageData* imageData)
 {
     imageData->ffCopyMagickString(imageData->imageInfo->magick, "RGBA", 5);
 
@@ -279,7 +384,7 @@ static bool printImageKitty(FFinstance* instance, FFLogoRequestData* requestData
         return false;
 
     #ifdef FF_HAVE_ZLIB
-        bool isCompressed = compressBlob(instance, &blob, &length);
+        bool isCompressed = compressBlob(&blob, &length);
     #else
         bool isCompressed = false;
     #endif
@@ -289,8 +394,7 @@ static bool printImageKitty(FFinstance* instance, FFLogoRequestData* requestData
     if(!checkAllocationResult(chars, length))
         return false;
 
-    FFstrbuf result;
-    ffStrbufInitA(&result, (uint32_t) (length + 1024));
+    FF_STRBUF_AUTO_DESTROY result = ffStrbufCreateA((uint32_t) (length + 1024));
 
     const char* currentPos = chars;
     size_t remainingLength = length;
@@ -302,18 +406,17 @@ static bool printImageKitty(FFinstance* instance, FFLogoRequestData* requestData
     while(remainingLength > 0)
         appendKittyChunk(&result, &currentPos, &remainingLength, true);
 
-    printImagePixels(instance, requestData, &result, isCompressed ? FF_CACHE_FILE_KITTY_COMPRESSED : FF_CACHE_FILE_KITTY_UNCOMPRESSED);
+    printImagePixels(requestData, &result, isCompressed ? FF_CACHE_FILE_KITTY_COMPRESSED : FF_CACHE_FILE_KITTY_UNCOMPRESSED);
 
-    ffStrbufDestroy(&result);
     free(chars);
     return true;
 }
 
 #ifdef FF_HAVE_CHAFA
 #include <chafa.h>
-static bool printImageChafa(FFinstance* instance, FFLogoRequestData* requestData, const ImageData* imageData)
+static bool printImageChafa(FFLogoRequestData* requestData, const ImageData* imageData)
 {
-    FF_LIBRARY_LOAD(chafa, &instance->config.libChafa, false,
+    FF_LIBRARY_LOAD(chafa, &instance.config.library.libChafa, false,
         "libchafa" FF_LIBRARY_EXTENSION, 1,
         "libchafa-0" FF_LIBRARY_EXTENSION, -1 // Required for Windows
     )
@@ -337,36 +440,36 @@ static bool printImageChafa(FFinstance* instance, FFLogoRequestData* requestData
 
     ChafaSymbolMap* symbolMap = ffchafa_symbol_map_new();
     GError* error = NULL;
-    if(!ffchafa_symbol_map_apply_selectors(symbolMap, instance->config.logo.chafaSymbols.chars, &error))
+    if(!ffchafa_symbol_map_apply_selectors(symbolMap, instance.config.logo.chafaSymbols.chars, &error))
         fputs(error->message, stderr);
 
     ChafaCanvasConfig* canvasConfig = ffchafa_canvas_config_new();
     ffchafa_canvas_config_set_geometry(canvasConfig, (gint) requestData->logoCharacterWidth, (gint) requestData->logoCharacterHeight);
     ffchafa_canvas_config_set_symbol_map(canvasConfig, symbolMap);
 
-    if(instance->config.logo.chafaFgOnly)
+    if(instance.config.logo.chafaFgOnly)
     {
         FF_LIBRARY_LOAD_SYMBOL_LAZY(chafa, chafa_canvas_config_set_fg_only_enabled);
         if(ffchafa_canvas_config_set_fg_only_enabled)
             ffchafa_canvas_config_set_fg_only_enabled(canvasConfig, true);
     }
-    if(instance->config.logo.chafaCanvasMode < CHAFA_CANVAS_MODE_MAX)
+    if(instance.config.logo.chafaCanvasMode < CHAFA_CANVAS_MODE_MAX)
     {
         FF_LIBRARY_LOAD_SYMBOL_LAZY(chafa, chafa_canvas_config_set_canvas_mode);
         if(ffchafa_canvas_config_set_canvas_mode)
-            ffchafa_canvas_config_set_canvas_mode(canvasConfig, (ChafaCanvasMode) instance->config.logo.chafaCanvasMode);
+            ffchafa_canvas_config_set_canvas_mode(canvasConfig, (ChafaCanvasMode) instance.config.logo.chafaCanvasMode);
     }
-    if(instance->config.logo.chafaColorSpace < CHAFA_COLOR_SPACE_MAX)
+    if(instance.config.logo.chafaColorSpace < CHAFA_COLOR_SPACE_MAX)
     {
         FF_LIBRARY_LOAD_SYMBOL_LAZY(chafa, chafa_canvas_config_set_color_space)
         if(ffchafa_canvas_config_set_color_space)
-            ffchafa_canvas_config_set_color_space(canvasConfig, (ChafaColorSpace) instance->config.logo.chafaColorSpace);
+            ffchafa_canvas_config_set_color_space(canvasConfig, (ChafaColorSpace) instance.config.logo.chafaColorSpace);
     }
-    if(instance->config.logo.chafaDitherMode < CHAFA_DITHER_MODE_MAX)
+    if(instance.config.logo.chafaDitherMode < CHAFA_DITHER_MODE_MAX)
     {
         FF_LIBRARY_LOAD_SYMBOL_LAZY(chafa, chafa_canvas_config_set_dither_mode)
         if(ffchafa_canvas_config_set_dither_mode)
-            ffchafa_canvas_config_set_dither_mode(canvasConfig, (ChafaDitherMode) instance->config.logo.chafaDitherMode);
+            ffchafa_canvas_config_set_dither_mode(canvasConfig, (ChafaDitherMode) instance.config.logo.chafaDitherMode);
     }
 
     ChafaCanvas* canvas = ffchafa_canvas_new(canvasConfig);
@@ -385,7 +488,7 @@ static bool printImageChafa(FFinstance* instance, FFLogoRequestData* requestData
     result.length = (uint32_t) str->len;
     result.chars = str->str;
 
-    ffLogoPrintChars(instance, result.chars, false);
+    ffLogoPrintChars(result.chars, false);
     writeCacheStrbuf(requestData, &result, FF_CACHE_FILE_CHAFA);
 
     // FIXME: These functions must be imported from `libglib` dlls on Windows
@@ -407,7 +510,7 @@ static bool printImageChafa(FFinstance* instance, FFLogoRequestData* requestData
 }
 #endif
 
-FFLogoImageResult ffLogoPrintImageImpl(FFinstance* instance, FFLogoRequestData* requestData, const FFIMData* imData)
+FFLogoImageResult ffLogoPrintImageImpl(FFLogoRequestData* requestData, const FFIMData* imData)
 {
     FF_LIBRARY_LOAD_SYMBOL(imData->library, MagickCoreGenesis, FF_LOGO_IMAGE_RESULT_INIT_ERROR);
     FF_LIBRARY_LOAD_SYMBOL(imData->library, MagickCoreTerminus, FF_LOGO_IMAGE_RESULT_INIT_ERROR);
@@ -442,7 +545,7 @@ FFLogoImageResult ffLogoPrintImageImpl(FFinstance* instance, FFLogoRequestData*
     }
 
     //+1, because we need to copy the null byte too
-    imageData.ffCopyMagickString(imageInfoIn->filename, instance->config.logo.source.chars, instance->config.logo.source.length + 1);
+    imageData.ffCopyMagickString(imageInfoIn->filename, instance.config.logo.source.chars, instance.config.logo.source.length + 1);
 
     imageData.image = ffReadImage(imageInfoIn, imageData.exceptionInfo);
     ffDestroyImageInfo(imageInfoIn);
@@ -463,8 +566,8 @@ FFLogoImageResult ffLogoPrintImageImpl(FFinstance* instance, FFLogoRequestData*
     else if(requestData->logoPixelHeight == 0)
         requestData->logoPixelHeight = (uint32_t) ((double) imageData.image->rows / (double) imageData.image->columns * requestData->logoPixelWidth);
 
-    requestData->logoCharacterWidth = simpleCeil((double) requestData->logoPixelWidth / requestData->characterPixelWidth);
-    requestData->logoCharacterHeight = simpleCeil((double) requestData->logoPixelHeight / requestData->characterPixelHeight);
+    requestData->logoCharacterWidth = (uint32_t) ceil((double) requestData->logoPixelWidth / requestData->characterPixelWidth);
+    requestData->logoCharacterHeight = (uint32_t) ceil((double) requestData->logoPixelHeight / requestData->characterPixelHeight);
 
     if(requestData->logoPixelWidth == 0 || requestData->logoPixelHeight == 0 || requestData->logoCharacterWidth == 0 || requestData->logoCharacterHeight == 0)
     {
@@ -497,13 +600,13 @@ FFLogoImageResult ffLogoPrintImageImpl(FFinstance* instance, FFLogoRequestData*
     if(requestData->type == FF_LOGO_TYPE_IMAGE_CHAFA)
     {
         #ifdef FF_HAVE_CHAFA
-            printSuccessful = printImageChafa(instance, requestData, &imageData);
+            printSuccessful = printImageChafa(requestData, &imageData);
         #endif
     }
     else if(requestData->type == FF_LOGO_TYPE_IMAGE_KITTY)
-        printSuccessful = printImageKitty(instance, requestData, &imageData);
+        printSuccessful = printImageKitty(requestData, &imageData);
     else if(requestData->type == FF_LOGO_TYPE_IMAGE_SIXEL)
-        printSuccessful = printImageSixel(instance, requestData, &imageData);
+        printSuccessful = printImageSixel(requestData, &imageData);
 
     ffDestroyImageInfo(imageData.imageInfo);
     ffDestroyImage(imageData.image);
@@ -517,7 +620,11 @@ static int getCacheFD(FFLogoRequestData* requestData, const char* fileName)
 {
     uint32_t cacheDirLength = requestData->cacheDir.length;
     ffStrbufAppendS(&requestData->cacheDir, fileName);
-    int fd = open(requestData->cacheDir.chars, O_RDONLY);
+    int fd = open(requestData->cacheDir.chars, O_RDONLY
+        #ifdef O_CLOEXEC
+            | O_CLOEXEC
+        #endif
+    );
     ffStrbufSubstrBefore(&requestData->cacheDir, cacheDirLength);
     return fd;
 }
@@ -532,47 +639,36 @@ static void readCachedStrbuf(FFLogoRequestData* requestData, FFstrbuf* result, c
 
 static uint32_t readCachedUint32(FFLogoRequestData* requestData, const char* cacheFileName)
 {
-    FFstrbuf content;
-    ffStrbufInit(&content);
+    FF_STRBUF_AUTO_DESTROY content = ffStrbufCreate();
     readCachedStrbuf(requestData, &content, cacheFileName);
 
     uint32_t result = 0;
 
     if(content.length != sizeof(result))
-    {
-        ffStrbufDestroy(&content);
         return 0;
-    }
 
     memcpy(&result, content.chars, sizeof(result));
 
-    ffStrbufDestroy(&content);
-
     return result;
 }
 
-static bool printCachedChars(FFinstance* instance, FFLogoRequestData* requestData)
+static bool printCachedChars(FFLogoRequestData* requestData)
 {
-    FFstrbuf content;
-    ffStrbufInitA(&content, 32768);
+    FF_STRBUF_AUTO_DESTROY content = ffStrbufCreateA(32768);
 
     if(requestData->type == FF_LOGO_TYPE_IMAGE_CHAFA)
         readCachedStrbuf(requestData, &content, FF_CACHE_FILE_CHAFA);
 
     if(content.length == 0)
-    {
-        ffStrbufDestroy(&content);
         return false;
-    }
 
-    ffLogoPrintChars(instance, content.chars, false);
-    ffStrbufDestroy(&content);
+    ffLogoPrintChars(content.chars, false);
     return true;
 }
 
-static bool printCachedPixel(FFinstance* instance, FFLogoRequestData* requestData)
+static bool printCachedPixel(FFLogoRequestData* requestData)
 {
-    requestData->logoCharacterWidth = instance->config.logo.width;
+    requestData->logoCharacterWidth = instance.config.logo.width;
     if(requestData->logoCharacterWidth == 0)
     {
         requestData->logoCharacterWidth = readCachedUint32(requestData, FF_CACHE_FILE_WIDTH);
@@ -580,7 +676,7 @@ static bool printCachedPixel(FFinstance* instance, FFLogoRequestData* requestDat
             return false;
     }
 
-    requestData->logoCharacterHeight = instance->config.logo.height;
+    requestData->logoCharacterHeight = instance.config.logo.height;
     if(requestData->logoCharacterHeight == 0)
     {
         requestData->logoCharacterHeight = readCachedUint32(requestData, FF_CACHE_FILE_HEIGHT);
@@ -601,8 +697,8 @@ static bool printCachedPixel(FFinstance* instance, FFLogoRequestData* requestDat
     if(fd == -1)
         return false;
 
-    ffPrintCharTimes('\n', instance->config.logo.paddingTop);
-    ffPrintCharTimes(' ', instance->config.logo.paddingLeft);
+    ffPrintCharTimes('\n', instance.config.logo.paddingTop);
+    ffPrintCharTimes(' ', instance.config.logo.paddingLeft);
     fflush(stdout);
 
     char buffer[32768];
@@ -612,21 +708,20 @@ static bool printCachedPixel(FFinstance* instance, FFLogoRequestData* requestDat
 
     close(fd);
 
-    instance->state.logoWidth = requestData->logoCharacterWidth + instance->config.logo.paddingLeft + instance->config.logo.paddingRight;
-    instance->state.logoHeight = requestData->logoCharacterHeight + instance->config.logo.paddingTop;
+    instance.state.logoWidth = requestData->logoCharacterWidth + instance.config.logo.paddingLeft + instance.config.logo.paddingRight;
+    instance.state.logoHeight = requestData->logoCharacterHeight + instance.config.logo.paddingTop;
 
     //Go to upper left corner
-    fputs("\033[9999999D", stdout);
-    printf("\033[%uA", instance->state.logoHeight);
+    printf("\e[1G\e[%uA", instance.state.logoHeight);
     return true;
 }
 
-static bool printCached(FFinstance* instance, FFLogoRequestData* requestData)
+static bool printCached(FFLogoRequestData* requestData)
 {
     if(requestData->type == FF_LOGO_TYPE_IMAGE_CHAFA)
-        return printCachedChars(instance, requestData);
+        return printCachedChars(requestData);
     else
-        return printCachedPixel(instance, requestData);
+        return printCachedPixel(requestData);
 }
 
 static bool getCharacterPixelDimensions(FFLogoRequestData* requestData)
@@ -666,7 +761,7 @@ static bool getCharacterPixelDimensions(FFLogoRequestData* requestData)
     return requestData->characterPixelWidth > 1.0 && requestData->characterPixelHeight > 1.0;
 }
 
-static bool printImageIfExistsSlowPath(FFinstance* instance, FFLogoType type, bool printError)
+static bool printImageIfExistsSlowPath(FFLogoType type, bool printError)
 {
     FFLogoRequestData requestData;
     requestData.type = type;
@@ -674,7 +769,7 @@ static bool printImageIfExistsSlowPath(FFinstance* instance, FFLogoType type, bo
     requestData.characterPixelHeight = 1;
 
     if(
-        (type != FF_LOGO_TYPE_IMAGE_CHAFA || instance->config.logo.width == 0 || instance->config.logo.height == 0) &&
+        (type != FF_LOGO_TYPE_IMAGE_CHAFA || instance.config.logo.width == 0 || instance.config.logo.height == 0) &&
         !getCharacterPixelDimensions(&requestData)
     ) {
         if(printError)
@@ -682,15 +777,15 @@ static bool printImageIfExistsSlowPath(FFinstance* instance, FFLogoType type, bo
         return false;
     }
 
-    requestData.logoPixelWidth = simpleCeil((double) instance->config.logo.width * requestData.characterPixelWidth);
-    requestData.logoPixelHeight = simpleCeil((double) instance->config.logo.height * requestData.characterPixelHeight);
+    requestData.logoPixelWidth = (uint32_t) ceil((double) instance.config.logo.width * requestData.characterPixelWidth);
+    requestData.logoPixelHeight = (uint32_t) ceil((double) instance.config.logo.height * requestData.characterPixelHeight);
 
     ffStrbufInit(&requestData.cacheDir);
-    ffStrbufAppend(&requestData.cacheDir, &instance->state.platform.cacheDir);
+    ffStrbufAppend(&requestData.cacheDir, &instance.state.platform.cacheDir);
     ffStrbufAppendS(&requestData.cacheDir, "fastfetch/images");
 
     ffStrbufEnsureFree(&requestData.cacheDir, PATH_MAX);
-    if(realpath(instance->config.logo.source.chars, requestData.cacheDir.chars + requestData.cacheDir.length) == NULL)
+    if(realpath(instance.config.logo.source.chars, requestData.cacheDir.chars + requestData.cacheDir.length) == NULL)
     {
         //We can safely return here, because if realpath failed, we surely won't be able to read the file
         ffStrbufDestroy(&requestData.cacheDir);
@@ -700,13 +795,9 @@ static bool printImageIfExistsSlowPath(FFinstance* instance, FFLogoType type, bo
     }
     ffStrbufRecalculateLength(&requestData.cacheDir);
     ffStrbufEnsureEndsWithC(&requestData.cacheDir, '/');
+    ffStrbufAppendF(&requestData.cacheDir, "%u*%u/", requestData.logoPixelWidth, requestData.logoPixelHeight);
 
-    ffStrbufAppendF(&requestData.cacheDir, "%u", requestData.logoPixelWidth);
-    ffStrbufAppendC(&requestData.cacheDir, '*');
-    ffStrbufAppendF(&requestData.cacheDir, "%u", requestData.logoPixelHeight);
-    ffStrbufAppendC(&requestData.cacheDir, '/');
-
-    if(!instance->config.recache && printCached(instance, &requestData))
+    if(!instance.config.logo.recache && printCached(&requestData))
     {
         ffStrbufDestroy(&requestData.cacheDir);
         return true;
@@ -715,12 +806,12 @@ static bool printImageIfExistsSlowPath(FFinstance* instance, FFLogoType type, bo
     FFLogoImageResult result = FF_LOGO_IMAGE_RESULT_INIT_ERROR;
 
     #ifdef FF_HAVE_IMAGEMAGICK7
-        result = ffLogoPrintImageIM7(instance, &requestData);
+        result = ffLogoPrintImageIM7(&requestData);
     #endif
 
     #ifdef FF_HAVE_IMAGEMAGICK6
         if(result == FF_LOGO_IMAGE_RESULT_INIT_ERROR)
-            result = ffLogoPrintImageIM6(instance, &requestData);
+            result = ffLogoPrintImageIM6(&requestData);
     #endif
 
     ffStrbufDestroy(&requestData.cacheDir);
@@ -741,25 +832,20 @@ static bool printImageIfExistsSlowPath(FFinstance* instance, FFLogoType type, bo
 
 #endif //FF_HAVE_IMAGEMAGICK{6, 7}
 
-bool ffLogoPrintImageIfExists(FFinstance* instance, FFLogoType type, bool printError)
+bool ffLogoPrintImageIfExists(FFLogoType type, bool printError)
 {
-    if(!ffPathExists(instance->config.logo.source.chars, FF_PATHTYPE_FILE))
+    if(!ffPathExists(instance.config.logo.source.chars, FF_PATHTYPE_FILE))
     {
         if(printError)
-            fprintf(stderr, "Logo: Image source \"%s\" does not exist\n", instance->config.logo.source.chars);
+            fprintf(stderr, "Logo: Image source \"%s\" does not exist\n", instance.config.logo.source.chars);
         return false;
     }
 
     if(type == FF_LOGO_TYPE_IMAGE_ITERM)
-        return printImageIterm(instance);
+        return printImageIterm();
 
-    if(
-        type == FF_LOGO_TYPE_IMAGE_KITTY &&
-        ffStrbufEndsWithIgnCaseS(&instance->config.logo.source, ".png") &&
-        instance->config.logo.width &&
-        instance->config.logo.height
-    )
-        return printImageKittyDirect(instance);
+    if(type == FF_LOGO_TYPE_IMAGE_KITTY_DIRECT)
+        return printImageKittyDirect();
 
     #if !defined(FF_HAVE_CHAFA)
         if(type == FF_LOGO_TYPE_IMAGE_CHAFA)
@@ -775,6 +861,6 @@ bool ffLogoPrintImageIfExists(FFinstance* instance, FFLogoType type, bool printE
             fputs("Logo: Image Magick support is not compiled in\n", stderr);
         return false;
     #else
-        return printImageIfExistsSlowPath(instance, type, printError);
+        return printImageIfExistsSlowPath(type, printError);
     #endif
 }
